@@ -1,19 +1,22 @@
 #
-# Copyright (c) 2022-2023, NVIDIA CORPORATION. All rights reserved.
+# Copyright (c) 2022-2025, NVIDIA CORPORATION. All rights reserved.
 #
 
-from __future__ import annotations  # pylint: disable=unused-variable
+from __future__ import annotations
 
 import json
 import logging
+import os
 from pathlib import Path
 import time
-from typing import Dict, List, NewType, Iterable
+from typing import Dict, List, Iterable, Union, Optional
 import requests
+from requests.structures import CaseInsensitiveDict
 
 from aistore.sdk.ais_source import AISSource
-from aistore.sdk.etl_const import DEFAULT_ETL_TIMEOUT
-from aistore.sdk.object_iterator import ObjectIterator
+from aistore.sdk.etl.etl_const import DEFAULT_ETL_TIMEOUT
+from aistore.sdk.obj.object_iterator import ObjectIterator
+from aistore.sdk.etl import ETLConfig
 from aistore.sdk.const import (
     ACT_COPY_BCK,
     ACT_CREATE_BCK,
@@ -32,7 +35,6 @@ from aistore.sdk.const import (
     HTTP_METHOD_HEAD,
     HTTP_METHOD_POST,
     MSGPACK_CONTENT_TYPE,
-    PROVIDER_AIS,
     QPARAM_BCK_TO,
     QPARAM_BSUMM_REMOTE,
     QPARAM_FLT_PRESENCE,
@@ -44,7 +46,11 @@ from aistore.sdk.const import (
     STATUS_ACCEPTED,
     STATUS_OK,
     STATUS_PARTIAL_CONTENT,
+    DEFAULT_JOB_POLL_TIME,
 )
+from aistore.sdk.enums import FLTPresence
+from aistore.sdk.provider import Provider
+from aistore.sdk.dataset.dataset_config import DatasetConfig
 
 from aistore.sdk.errors import (
     InvalidBckProvider,
@@ -54,7 +60,7 @@ from aistore.sdk.errors import (
 )
 from aistore.sdk.multiobj import ObjectGroup, ObjectRange
 from aistore.sdk.request_client import RequestClient
-from aistore.sdk.object import Object
+from aistore.sdk.obj.object import Object, BucketDetails
 from aistore.sdk.types import (
     ActionMsg,
     BucketEntry,
@@ -69,11 +75,10 @@ from aistore.sdk.types import (
 )
 from aistore.sdk.list_object_flag import ListObjectFlag
 from aistore.sdk.utils import validate_directory, get_file_size
-
-Header = NewType("Header", requests.structures.CaseInsensitiveDict)
+from aistore.sdk.obj.object_props import ObjectProps
 
 
-# pylint: disable=unused-variable,too-many-public-methods
+# pylint: disable=too-many-public-methods,too-many-lines
 class Bucket(AISSource):
     """
     A class representing a bucket that contains user data.
@@ -81,37 +86,42 @@ class Bucket(AISSource):
     Args:
         client (RequestClient): Client for interfacing with AIS cluster
         name (str): name of bucket
-        provider (str, optional): Provider of bucket (one of "ais", "aws", "gcp", ...), defaults to "ais"
+        provider (str or Provider, optional): Provider of bucket (one of "ais", "aws", "gcp", ...), defaults to "ais"
         namespace (Namespace, optional): Namespace of bucket, defaults to None
     """
 
     def __init__(
         self,
         name: str,
-        client: RequestClient = None,
-        provider: str = PROVIDER_AIS,
-        namespace: Namespace = None,
+        client: RequestClient,
+        provider: Union[Provider, str] = Provider.AIS,
+        namespace: Optional[Namespace] = None,
     ):
         self._client = client
         self._name = name
-        self._provider = provider
+        self._provider = Provider.parse(provider)
         self._namespace = namespace
-        self._qparam = {QPARAM_PROVIDER: provider}
-        if self.namespace:
-            self._qparam[QPARAM_NAMESPACE] = namespace.get_path()
+        self._qparam = {QPARAM_PROVIDER: self.provider.value}
+        if self._namespace:
+            self._qparam[QPARAM_NAMESPACE] = self._namespace.get_path()
 
     @property
     def client(self) -> RequestClient:
-        """The client bound to this bucket."""
+        """The client used by this bucket."""
         return self._client
 
+    @client.setter
+    def client(self, client):
+        """Update the client used by this bucket."""
+        self._client = client
+
     @property
     def qparam(self) -> Dict:
         """Default query parameters to use with API calls from this bucket."""
         return self._qparam
 
     @property
-    def provider(self) -> str:
+    def provider(self) -> Provider:
         """The provider for this bucket."""
         return self._provider
 
@@ -121,22 +131,45 @@ class Bucket(AISSource):
         return self._name
 
     @property
-    def namespace(self) -> Namespace:
+    def namespace(self) -> Optional[Namespace]:
         """The namespace for this bucket."""
         return self._namespace
 
-    def list_urls(self, prefix: str = "", etl_name: str = None) -> Iterable[str]:
+    def list_urls(
+        self, prefix: str = "", etl: Optional[ETLConfig] = None
+    ) -> Iterable[str]:
         """
-            Get an iterator of full URLs to every object in this bucket matching the prefix
-        Args:
-            prefix (str, optional): Limit objects selected by a given string prefix
-            etl_name (str, optional): ETL to include in URLs
+        Generates full URLs for all objects in the bucket that match the specified prefix.
 
-        Returns:
-            Iterator of all object URLs matching the prefix
+        Args:
+            prefix (str, optional): A string prefix to filter objects. Only objects with names starting
+                with this prefix will be included. Defaults to an empty string (no filtering).
+            etl (Optional[ETLConfig], optional): An optional ETL configuration. If provided, the URLs
+                will include ETL processing parameters. Defaults to None.
+
+        Yields:
+            str: Full URLs of all objects matching the prefix.
         """
         for entry in self.list_objects_iter(prefix=prefix, props="name"):
-            yield self.object(entry.name).get_url(etl_name=etl_name)
+            yield self.object(entry.name).get_url(etl=etl)
+
+    def list_all_objects_iter(
+        self, prefix: str = "", props: str = "name,size"
+    ) -> Iterable[Object]:
+        """
+        Implementation of the abstract method from AISSource that provides an iterator
+        of all the objects in this bucket matching the specified prefix.
+
+        Args:
+            prefix (str, optional): Limit objects selected by a given string prefix
+            props (str, optional): Comma-separated list of object properties to return. Default value is "name,size".
+                Properties: "name", "size", "atime", "version", "checksum", "target_url", "copies".
+
+        Yields:
+            Object: Objects matching the prefix with the specified properties.
+        """
+        for entry in self.list_objects_iter(prefix=prefix, props=props):
+            yield self.object(entry.name, entry.generate_object_props())
 
     def create(self, exist_ok=False):
         """
@@ -212,8 +245,8 @@ class Bucket(AISSource):
         """
         self._verify_ais_bucket()
         params = self.qparam.copy()
-        params[QPARAM_BCK_TO] = Bucket(
-            name=to_bck_name, namespace=self.namespace
+        params[QPARAM_BCK_TO] = BucketModel(
+            name=to_bck_name, namespace=self.namespace, provider=self.provider.value
         ).get_path()
         resp = self.make_request(HTTP_METHOD_POST, ACT_MOVE_BCK, params=params)
         self._name = to_bck_name
@@ -242,7 +275,7 @@ class Bucket(AISSource):
         params[QPARAM_KEEP_REMOTE] = str(keep_md)
         self.make_request(HTTP_METHOD_DELETE, ACT_EVICT_REMOTE_BCK, params=params)
 
-    def head(self) -> Header:
+    def head(self) -> CaseInsensitiveDict:
         """
         Requests bucket properties.
 
@@ -263,7 +296,7 @@ class Bucket(AISSource):
             params=self.qparam,
         ).headers
 
-    # pylint: disable=too-many-arguments
+    # pylint: disable=too-many-arguments,too-many-positional-arguments
     def summary(
         self,
         uuid: str = "",
@@ -282,6 +315,7 @@ class Bucket(AISSource):
             present (bool): If True, summary entails present entities. Defaults to True.
 
         Raises:
+            UnexpectedHTTPStatusCode: If the response status code is not as expected
             requests.ConnectionError: Connection error
             requests.ConnectionTimeout: Timed out connecting to AIStore
             requests.exceptions.HTTPError: Service unavailable
@@ -298,7 +332,7 @@ class Bucket(AISSource):
             HTTP_METHOD_GET,
             ACT_SUMMARY_BCK,
             params=self.qparam,
-            value=bsumm_ctrl_msg.dict(),
+            value=bsumm_ctrl_msg.model_dump(),
         )
 
         # Initial response status code should be 202
@@ -310,8 +344,8 @@ class Bucket(AISSource):
         # Update the uuid in the control message
         bsumm_ctrl_msg.uuid = job_id
 
-        # Sleep and request frequency in sec (starts at 200 ms)
-        sleep_time = 0.2
+        # Sleep and request frequency in sec (starts at 0.2s)
+        sleep_time = DEFAULT_JOB_POLL_TIME
 
         # Poll async task for http.StatusOK completion
         while True:
@@ -319,44 +353,54 @@ class Bucket(AISSource):
                 HTTP_METHOD_GET,
                 ACT_SUMMARY_BCK,
                 params=self.qparam,
-                value=bsumm_ctrl_msg.dict(),
+                value=bsumm_ctrl_msg.model_dump(),
             )
 
             # If task completed successfully, break the loop
             if resp.status_code == STATUS_OK:
                 break
+            # If status code received is neither STATUS_ACCEPTED nor STATUS_PARTIAL_CONTENT, raise an exception
+            if resp.status_code not in (STATUS_ACCEPTED, STATUS_PARTIAL_CONTENT):
+                raise UnexpectedHTTPStatusCode(
+                    [STATUS_OK, STATUS_ACCEPTED, STATUS_PARTIAL_CONTENT],
+                    resp.status_code,
+                )
+
             # If task is still running, wait for some time and try again
-            if resp.status_code == STATUS_ACCEPTED:
-                time.sleep(sleep_time)
+            time.sleep(sleep_time)
+            if resp.status_code != STATUS_PARTIAL_CONTENT:
                 sleep_time = min(
                     10, sleep_time * 1.5
                 )  # Increase sleep_time by 50%, but don't exceed 10 seconds
-            # Otherwise, if status code received is neither STATUS_OK or STATUS_ACCEPTED, raise an exception
-            else:
-                raise UnexpectedHTTPStatusCode(
-                    [STATUS_OK, STATUS_ACCEPTED], resp.status_code
-                )
 
         return json.loads(resp.content.decode("utf-8"))[0]
 
-    def info(self, flt_presence: int = 0, bsumm_remote: bool = True):
+    def info(
+        self,
+        flt_presence: int = FLTPresence.FLT_EXISTS,
+        bsumm_remote: bool = True,
+        prefix: str = "",
+    ):
         """
         Returns bucket summary and information/properties.
 
         Args:
-            flt_presence (int): Describes the presence of buckets and objects with respect to their existence
-                                or non-existence in the AIS cluster. Defaults to 0.
-
-                                Expected values are:
-                                0 - (object | bucket) exists inside and/or outside cluster
-                                1 - same as 0 but no need to return summary
-                                2 - bucket: is present | object: present and properly located
-                                3 - same as 2 but no need to return summary
-                                4 - objects: present anywhere/anyhow _in_ the cluster as: replica, ec-slices, misplaced
-                                5 - not present - exists _outside_ cluster
+            flt_presence (FLTPresence): Describes the presence of buckets and objects with respect to their existence
+                                or non-existence in the AIS cluster using the enum FLTPresence. Defaults to
+                                value FLT_EXISTS and values are:
+                                FLT_EXISTS - object or bucket exists inside and/or outside cluster
+                                FLT_EXISTS_NO_PROPS - same as FLT_EXISTS but no need to return summary
+                                FLT_PRESENT - bucket is present or object is present and properly
+                                located
+                                FLT_PRESENT_NO_PROPS - same as FLT_PRESENT but no need to return summary
+                                FLT_PRESENT_CLUSTER - objects present anywhere/how in
+                                the cluster as replica, ec-slices, misplaced
+                                FLT_EXISTS_OUTSIDE - not present; exists outside cluster
             bsumm_remote (bool): If True, returned bucket info will include remote objects as well
+            prefix (str): Only include objects with the given prefix in the bucket
 
         Raises:
+            UnexpectedHTTPStatusCode: If the response status code is not as expected
             requests.ConnectionError: Connection error
             requests.ConnectionTimeout: Timed out connecting to AIStore
             requests.exceptions.HTTPError: Service unavailable
@@ -365,16 +409,24 @@ class Bucket(AISSource):
             ValueError: `flt_presence` is not one of the expected values
             aistore.sdk.errors.AISError: All other types of errors with AIStore
         """
-        if flt_presence not in range(6):
-            raise ValueError("`flt_presence` must be one of 0, 1, 2, 3, 4, or 5.")
+        try:
+            flt_presence = int(FLTPresence(flt_presence))
+        except Exception as err:
+            raise ValueError(
+                "`flt_presence` must be in values of enum FLTPresence"
+            ) from err
 
         params = self.qparam.copy()
         params.update({QPARAM_FLT_PRESENCE: flt_presence})
         params[QPARAM_BSUMM_REMOTE] = bsumm_remote
 
+        path = f"{URL_PATH_BUCKETS}/{self.name}"
+        if prefix:
+            path += f"/{prefix}"
+
         response = self.client.request(
             HTTP_METHOD_HEAD,
-            path=f"{URL_PATH_BUCKETS}/{self.name}",
+            path=path,
             params=params,
         )
 
@@ -382,6 +434,8 @@ class Bucket(AISSource):
         uuid = response.headers.get(HEADER_XACTION_ID, "").strip('"')
         params[QPARAM_UUID] = uuid
 
+        result = {}
+
         # Initial response status code should be 202
         if response.status_code != int(STATUS_ACCEPTED):
             raise UnexpectedHTTPStatusCode([STATUS_ACCEPTED], response.status_code)
@@ -418,16 +472,18 @@ class Bucket(AISSource):
 
         return bucket_props, result
 
-    # pylint: disable=too-many-arguments
+    # pylint: disable=too-many-arguments,too-many-positional-arguments
     def copy(
         self,
         to_bck: Bucket,
         prefix_filter: str = "",
         prepend: str = "",
+        ext: Optional[Dict[str, str]] = None,
         dry_run: bool = False,
         force: bool = False,
         latest: bool = False,
         sync: bool = False,
+        num_workers: Optional[int] = 0,
     ) -> str:
         """
         Returns job ID that can be used later to check the status of the asynchronous operation.
@@ -436,11 +492,16 @@ class Bucket(AISSource):
             to_bck (Bucket): Destination bucket
             prefix_filter (str, optional): Only copy objects with names starting with this prefix
             prepend (str, optional): Value to prepend to the name of copied objects
+            ext (Dict[str, str], optional): Dict mapping each extension to the extension that will replace it
+                (e.g. {"jpg": "txt"})
             dry_run (bool, optional): Determines if the copy should actually
                 happen or not
             force (bool, optional): Override existing destination bucket
             latest (bool, optional): GET the latest object version from the associated remote bucket
             sync (bool, optional): synchronize destination bucket with its remote (e.g., Cloud or remote AIS) source
+            num_workers (int, optional): Number of concurrent workers for the copy job per target
+                - 0 (default): number of mountpaths
+                - -1: single thread, serial execution
 
         Returns:
             Job ID (as str) that can be used to check the status of the operation
@@ -453,21 +514,26 @@ class Bucket(AISSource):
             requests.RequestException: "There was an ambiguous exception that occurred while handling..."
             requests.ReadTimeout: Timed out receiving response from AIStore
         """
-        value = CopyBckMsg(
-            prefix=prefix_filter,
-            prepend=prepend,
-            dry_run=dry_run,
-            force=force,
-            latest=latest,
-            sync=sync,
+        value = TCBckMsg(
+            ext=ext,
+            num_workers=num_workers,
+            copy_msg=CopyBckMsg(
+                prefix=prefix_filter,
+                prepend=prepend,
+                force=force,
+                dry_run=dry_run,
+                latest=latest,
+                sync=sync,
+            ),
         ).as_dict()
+
         params = self.qparam.copy()
         params[QPARAM_BCK_TO] = to_bck.get_path()
         return self.make_request(
             HTTP_METHOD_POST, ACT_COPY_BCK, value=value, params=params
         ).text
 
-    # pylint: disable=too-many-arguments
+    # pylint: disable=too-many-arguments,too-many-positional-arguments
     def list_objects(
         self,
         prefix: str = "",
@@ -475,7 +541,7 @@ class Bucket(AISSource):
         page_size: int = 0,
         uuid: str = "",
         continuation_token: str = "",
-        flags: List[ListObjectFlag] = None,
+        flags: Optional[List[ListObjectFlag]] = None,
         target: str = "",
     ) -> BucketList:
         """
@@ -520,7 +586,7 @@ class Bucket(AISSource):
             flags=[] if flags is None else flags,
             target=target,
         ).as_dict()
-        action = ActionMsg(action=ACT_LIST, value=value).dict()
+        action = ActionMsg(action=ACT_LIST, value=value).model_dump()
 
         bucket_list = self.client.request_deserialize(
             HTTP_METHOD_GET,
@@ -540,7 +606,7 @@ class Bucket(AISSource):
         prefix: str = "",
         props: str = "",
         page_size: int = 0,
-        flags: List[ListObjectFlag] = None,
+        flags: Optional[List[ListObjectFlag]] = None,
         target: str = "",
     ) -> ObjectIterator:
         """
@@ -590,7 +656,7 @@ class Bucket(AISSource):
         prefix: str = "",
         props: str = "",
         page_size: int = 0,
-        flags: List[ListObjectFlag] = None,
+        flags: Optional[List[ListObjectFlag]] = None,
         target: str = "",
     ) -> List[BucketEntry]:
         """
@@ -644,7 +710,48 @@ class Bucket(AISSource):
             uuid = resp.uuid
         return obj_list
 
-    # pylint: disable=too-many-arguments
+    def list_archive(
+        self,
+        archive_obj_name: str,
+        include_archive_obj: bool = False,
+        props: str = "",
+        page_size: int = 0,
+    ) -> List[BucketEntry]:
+        """
+        List files contained in an archived object (*.tar, *.zip, *.tgz, etc.).
+
+        This is a convenience wrapper around `list_all_objects` that
+        automatically enables the `ARCH_DIR` list-flag so the cluster opens
+        the shard and returns its directory.
+
+        Args:
+            archive_obj_name (str): Object key of the shard inside this bucket
+                (e.g. `"my-archive.tar"`). Can include a prefix path.
+            include_archive_obj (bool, optional): If `True` the returned
+                list includes the parent archive object itself. When
+                `False` (default) only the entries *inside* the shard are
+                returned.
+            props (str, optional): Comma-separated list of object properties to
+                request. Defaults to `""` (no properties).
+            page_size (int, optional): Same meaning as in
+                `list_all_objects` – how many names per internal page.
+
+        Returns:
+            List[BucketEntry]: Entries representing the shard (optionally) and
+                every file stored inside it.
+        """
+        entries = self.list_all_objects(
+            prefix=archive_obj_name,
+            props=props,
+            page_size=page_size,
+            flags=[ListObjectFlag.ARCH_DIR],
+        )
+
+        if include_archive_obj:
+            return entries
+        return [e for e in entries if e.name != archive_obj_name]
+
+    # pylint: disable=too-many-arguments,too-many-positional-arguments,too-many-locals
     def transform(
         self,
         etl_name: str,
@@ -652,11 +759,14 @@ class Bucket(AISSource):
         timeout: str = DEFAULT_ETL_TIMEOUT,
         prefix_filter: str = "",
         prepend: str = "",
-        ext: Dict[str, str] = None,
+        ext: Optional[Dict[str, str]] = None,
         force: bool = False,
         dry_run: bool = False,
         latest: bool = False,
         sync: bool = False,
+        num_workers: Optional[int] = 0,
+        cont_on_err: bool = False,
+        etl_pipeline: Optional[List[str]] = None,
     ) -> str:
         """
         Visits all selected objects in the source bucket and for each object, puts the transformed
@@ -668,19 +778,28 @@ class Bucket(AISSource):
             timeout (str, optional): Timeout of the ETL job (e.g. 5m for 5 minutes)
             prefix_filter (str, optional): Only transform objects with names starting with this prefix
             prepend (str, optional): Value to prepend to the name of resulting transformed objects
-            ext (Dict[str, str], optional): dict of new extension followed by extension to be replaced
-                (i.e. {"jpg": "txt"})
+            ext (Dict[str, str], optional): Dict mapping each extension to the extension that will replace it
+                (e.g. {"jpg": "txt"})
             dry_run (bool, optional): determines if the copy should actually happen or not
             force (bool, optional): override existing destination bucket
             latest (bool, optional): GET the latest object version from the associated remote bucket
             sync (bool, optional): synchronize destination bucket with its remote (e.g., Cloud or remote AIS) source
+            num_workers (int, optional): Number of concurrent workers for the transformation job per target
+                - 0 (default): number of mountpaths
+                - -1: single thread, serial execution
+            cont_on_err: (bool): If True, continue processing objects even if some of them fail
+            etl_pipeline (List[str], optional): List of ETL names to be used for the transformation pipeline
 
         Returns:
             Job ID (as str) that can be used to check the status of the operation
         """
         value = TCBckMsg(
             ext=ext,
-            transform_msg=TransformBckMsg(etl_name=etl_name, timeout=timeout),
+            num_workers=num_workers,
+            cont_on_err=cont_on_err,
+            transform_msg=TransformBckMsg(
+                etl_name=etl_name, timeout=timeout, etl_pipeline=etl_pipeline
+            ),
             copy_msg=CopyBckMsg(
                 prefix=prefix_filter,
                 prepend=prepend,
@@ -699,11 +818,11 @@ class Bucket(AISSource):
 
     def put_files(
         self,
-        path: str,
+        path: Union[str, Path],
         prefix_filter: str = "",
         pattern: str = "*",
         basename: bool = False,
-        prepend: str = None,
+        prepend: Optional[str] = None,
         recursive: bool = False,
         dry_run: bool = False,
         verbose: bool = True,
@@ -712,9 +831,9 @@ class Bucket(AISSource):
         Puts files found in a given filepath as objects to a bucket in AIS storage.
 
         Args:
-            path (str): Local filepath, can be relative or absolute
+            path (str or Path): Local filepath, can be relative or absolute
             prefix_filter (str, optional): Only put files with names starting with this prefix
-            pattern (str, optional): Regex pattern to filter files
+            pattern (str, optional): Shell-style wildcard pattern to filter files
             basename (bool, optional): Whether to use the file names only as object names and omit the path information
             prepend (str, optional): Optional string to use as a prefix in the object name for all objects uploaded
                 No delimiter ("/", "-", etc.) is automatically applied between the prepend value and the object name
@@ -746,7 +865,7 @@ class Bucket(AISSource):
                 continue
             obj_name = self._get_uploaded_obj_name(file, path, basename, prepend)
             if not dry_run:
-                self.object(obj_name).put_file(str(file))
+                self.object(obj_name).get_writer().put_file(str(file))
             logger.info(
                 "%s File '%s' uploaded as object '%s' with size %s",
                 dry_run_prefix,
@@ -770,33 +889,41 @@ class Bucket(AISSource):
             return prepend + obj_name
         return obj_name
 
-    def object(self, obj_name: str) -> Object:
+    def object(
+        self,
+        obj_name: str,
+        props: Optional[ObjectProps] = None,
+    ) -> Object:
         """
         Factory constructor for an object in this bucket.
         Does not make any HTTP request, only instantiates an object in a bucket owned by the client.
 
         Args:
             obj_name (str): Name of object
+            props (ObjectProps, optional): Properties of the object, as updated by head(), optionally pre-initialized.
 
         Returns:
             The object created.
         """
+        details = BucketDetails(self.name, self.provider, self.qparam, self.get_path())
         return Object(
-            bucket=self,
+            client=self.client,
+            bck_details=details,
             name=obj_name,
+            props=props,
         )
 
     def objects(
         self,
-        obj_names: list = None,
-        obj_range: ObjectRange = None,
-        obj_template: str = None,
+        obj_names: Optional[List[str]] = None,
+        obj_range: Optional[ObjectRange] = None,
+        obj_template: Optional[str] = None,
     ) -> ObjectGroup:
         """
         Factory constructor for multiple objects belonging to this bucket.
 
         Args:
-            obj_names (list): Names of objects to include in the group
+            obj_names (List[str]): Names of objects to include in the group
             obj_range (ObjectRange): Range of objects to include in the group
             obj_template (str): String template defining objects to include in the group
 
@@ -814,8 +941,8 @@ class Bucket(AISSource):
         self,
         method: str,
         action: str,
-        value: dict = None,
-        params: dict = None,
+        value: Optional[Dict] = None,
+        params: Optional[Dict] = None,
     ) -> requests.Response:
         """
         Use the bucket's client to make a request to the bucket endpoint on the AIS server
@@ -823,7 +950,7 @@ class Bucket(AISSource):
         Args:
             method (str): HTTP method to use, e.g. POST/GET/DELETE
             action (str): Action string used to create an ActionMsg to pass to the server
-            value (dict): Additional value parameter to pass in the ActionMsg
+            value (dict, optional): Additional value parameter to pass in the ActionMsg
             params (dict, optional): Optional parameters to pass in the request
 
         Returns:
@@ -835,7 +962,7 @@ class Bucket(AISSource):
                 "Bucket requires a client to use functions. Try defining a client and accessing this bucket with "
                 "client.bucket()"
             )
-        json_val = ActionMsg(action=action, value=value).dict()
+        json_val = ActionMsg(action=action, value=value).model_dump()
         return self._client.request(
             method,
             path=f"{URL_PATH_BUCKETS}/{self.name}",
@@ -847,22 +974,21 @@ class Bucket(AISSource):
         """
         Verify the bucket provider is AIS
         """
-        if self.provider is not PROVIDER_AIS:
+        if self.provider.is_remote():
             raise InvalidBckProvider(self.provider)
 
     def verify_cloud_bucket(self):
         """
         Verify the bucket provider is a cloud provider
         """
-        if self.provider is PROVIDER_AIS:
+        if not self.provider.is_remote():
             raise InvalidBckProvider(self.provider)
 
     def get_path(self) -> str:
         """
         Get the path representation of this bucket
         """
-        namespace_path = self.namespace.get_path() if self.namespace else "@#"
-        return f"{ self.provider }/{ namespace_path }/{ self.name }/"
+        return self.as_model().get_path()
 
     def as_model(self) -> BucketModel:
         """
@@ -872,5 +998,32 @@ class Bucket(AISSource):
             BucketModel representation
         """
         return BucketModel(
-            name=self.name, namespace=self.namespace, provider=self.provider
+            name=self.name, namespace=self.namespace, provider=self.provider.value
         )
+
+    def write_dataset(
+        self,
+        config: DatasetConfig,
+        skip_missing: bool = True,
+        **kwargs,
+    ):
+        """
+        Write a dataset to a bucket in AIS in webdataset format using wds.ShardWriter. Logs the missing attributes
+
+        Args:
+            config (DatasetConfig): Configuration dict specifying how to process
+                and store each part of the dataset item
+            skip_missing (bool, optional): Skip samples that are missing one or more attributes, defaults to True
+            **kwargs (optional): Optional keyword arguments to pass to the ShardWriter
+        """
+
+        # Add the upload shard logic to the original post-processing function
+        original_post = kwargs.get("post", lambda path: None)
+
+        def combined_post_processing(shard_path):
+            original_post(shard_path)
+            self.object(shard_path).get_writer().put_file(shard_path)
+            os.unlink(shard_path)
+
+        kwargs["post"] = combined_post_processing
+        config.write_shards(skip_missing=skip_missing, **kwargs)
