@@ -1,8 +1,8 @@
 #
-# Copyright (c) 2023, NVIDIA CORPORATION. All rights reserved.
+# Copyright (c) 2023-2025, NVIDIA CORPORATION. All rights reserved.
 #
 import logging
-from typing import List, Iterable
+from typing import Dict, List, Iterable, Optional
 
 from aistore.sdk.ais_source import AISSource
 from aistore.sdk.const import (
@@ -16,7 +16,8 @@ from aistore.sdk.const import (
     ACT_TRANSFORM_OBJECTS,
     ACT_ARCHIVE_OBJECTS,
 )
-from aistore.sdk.etl_const import DEFAULT_ETL_TIMEOUT
+from aistore.sdk.etl.etl_const import DEFAULT_ETL_TIMEOUT
+from aistore.sdk.obj.object import Object
 from aistore.sdk.multiobj.object_names import ObjectNames
 from aistore.sdk.multiobj.object_range import ObjectRange
 from aistore.sdk.multiobj.object_template import ObjectTemplate
@@ -28,9 +29,10 @@ from aistore.sdk.types import (
     ArchiveMultiObj,
     PrefetchMsg,
 )
+from aistore.sdk.request_client import RequestClient
+from aistore.sdk.etl import ETLConfig
 
 
-# pylint: disable=unused-variable
 class ObjectGroup(AISSource):
     """
     A class representing multiple objects within the same bucket. Only one of obj_names, obj_range, or obj_template
@@ -46,9 +48,9 @@ class ObjectGroup(AISSource):
     def __init__(
         self,
         bck: "Bucket",
-        obj_names: list = None,
-        obj_range: ObjectRange = None,
-        obj_template: str = None,
+        obj_names: Optional[List] = None,
+        obj_range: Optional[ObjectRange] = None,
+        obj_template: Optional[str] = None,
     ):
         self.bck = bck
         num_args = sum(
@@ -65,21 +67,76 @@ class ObjectGroup(AISSource):
             self._obj_collection = obj_range
         elif obj_names:
             self._obj_collection = ObjectNames(obj_names)
-        else:
+        elif obj_template:
             self._obj_collection = ObjectTemplate(obj_template)
 
-    def list_urls(self, prefix: str = "", etl_name: str = None) -> Iterable[str]:
+    @property
+    def client(self) -> RequestClient:
+        """The client bound to the bucket used by the ObjectGroup."""
+        return self.bck.client
+
+    @client.setter
+    def client(self, client):
+        """Update the client bound to the bucket used by the ObjectGroup."""
+        self.bck.client = client
+
+    def _parse_job_ids(self, response_text: str) -> List[str]:
         """
-            Get an iterator of the full URL for every object in this group
+        Parse job ID response that may contain comma-separated UUIDs.
+        Used only for copy and archive operations which can return multiple UUIDs.
+
         Args:
-            prefix (str, optional): Limit objects selected by a given string prefix
-            etl_name (str, optional): ETL to include in URLs
+            response_text (str): Raw response text
 
         Returns:
-            Iterator of all object URLs in the group
+            List[str]: List of individual job IDs
+        """
+        return response_text.split(",") if response_text else []
+
+    def list_urls(
+        self, prefix: str = "", etl: Optional[ETLConfig] = None
+    ) -> Iterable[str]:
+        """
+        Implementation of the abstract method from AISSource that provides an iterator
+        of full URLs to every object in this bucket matching the specified prefix
+        Args:
+            prefix (str, optional): Limit objects selected by a given string prefix
+            etl (Optional[ETLConfig], optional): An optional ETL configuration. If provided, the URLs
+                will include ETL processing parameters. Defaults to None.
+
+        Yields:
+            str: Full URLs of objects in the group.
         """
         for obj_name in self._obj_collection:
-            yield self.bck.object(obj_name).get_url(etl_name=etl_name)
+            yield self.bck.object(obj_name).get_url(etl=etl)
+
+    def list_all_objects_iter(
+        self, prefix: str = "", props: str = "name,size"
+    ) -> Iterable[Object]:
+        """
+        Implementation of the abstract method from AISSource that provides an iterator
+        of all the objects in this bucket matching the specified prefix.
+
+        Args:
+            prefix (str, optional): Limit objects selected by a given string prefix
+            props (str, optional): By default, will include all object properties.
+                Pass in None to skip and avoid the extra API call.
+
+        Yields:
+            Object: Objects in the group matching the specified prefix.
+        """
+        for obj_name in self._obj_collection:
+
+            # If object does not start the prefix, skip it
+            if not obj_name.startswith(prefix):
+                continue
+
+            obj = self.bck.object(obj_name)
+
+            if props is not None:
+                obj.head()  # Updates the objects props as well
+
+            yield obj
 
     def delete(self):
         """
@@ -128,7 +185,13 @@ class ObjectGroup(AISSource):
             value=self._obj_collection.get_value(),
         ).text
 
-    def prefetch(self, latest: bool = False, continue_on_error: bool = False):
+    def prefetch(
+        self,
+        blob_threshold: Optional[int] = None,
+        num_workers: Optional[int] = None,
+        latest: bool = False,
+        continue_on_error: bool = False,
+    ):
         """
         Prefetches a list or range of objects in a bucket so that they are cached in AIS
         NOTE: only Cloud buckets can be prefetched.
@@ -136,6 +199,11 @@ class ObjectGroup(AISSource):
         Args:
             latest (bool, optional): GET the latest object version from the associated remote bucket
             continue_on_error (bool, optional): Whether to continue if there is an error prefetching a single object
+            blob_threshold (int, optional): Utilize built-in blob-downloader for remote objects
+                greater than the specified (threshold) size in bytes
+            num_workers (int, optional): Number of concurrent workers (readers). Defaults to the number of target
+                mountpaths if omitted or zero. A value of -1 indicates no workers at all (i.e., single-threaded
+                execution). Any positive value will be adjusted not to exceed the number of target CPUs.
 
         Raises:
             aistore.sdk.errors.AISError: All other types of errors with AIStore
@@ -155,6 +223,8 @@ class ObjectGroup(AISSource):
             object_selection=self._obj_collection.get_value(),
             continue_on_err=continue_on_error,
             latest=latest,
+            blob_threshold=blob_threshold,
+            num_workers=num_workers,
         ).as_dict()
 
         return self.bck.make_request(
@@ -163,7 +233,7 @@ class ObjectGroup(AISSource):
             value=value,
         ).text
 
-    # pylint: disable=too-many-arguments
+    # pylint: disable=too-many-arguments,too-many-positional-arguments
     def copy(
         self,
         to_bck: "Bucket",
@@ -173,7 +243,8 @@ class ObjectGroup(AISSource):
         force: bool = False,
         latest: bool = False,
         sync: bool = False,
-    ):
+        num_workers: int = None,
+    ) -> List[str]:
         """
         Copies a list or range of objects in a bucket
 
@@ -186,6 +257,9 @@ class ObjectGroup(AISSource):
                 (see "limited coexistence" and xact/xreg/xreg.go)
             latest (bool, optional): GET the latest object version from the associated remote bucket
             sync (bool, optional): synchronize destination bucket with its remote (e.g., Cloud or remote AIS) source
+            num_workers (int, optional): Number of concurrent workers (readers). Defaults to the number of target
+                mountpaths if omitted or zero. A value of -1 indicates no workers at all (i.e., single-threaded
+                execution). Any positive value will be adjusted not to exceed the number of target CPUs.
 
         Raises:
             aistore.sdk.errors.AISError: All other types of errors with AIStore
@@ -196,7 +270,7 @@ class ObjectGroup(AISSource):
             requests.ReadTimeout: Timed out receiving response from AIStore
 
         Returns:
-            Job ID (as str) that can be used to check the status of the operation
+            List[str]: List of job IDs that can be used to check the status of the operation
 
         """
         if dry_run:
@@ -216,26 +290,32 @@ class ObjectGroup(AISSource):
             tc_msg=TCBckMsg(copy_msg=copy_msg),
             object_selection=self._obj_collection.get_value(),
             continue_on_err=continue_on_error,
+            num_workers=num_workers,
         ).as_dict()
 
-        return self.bck.make_request(
+        response = self.bck.make_request(
             HTTP_METHOD_POST,
             ACT_COPY_OBJECTS,
             value=value,
-        ).text
+        )
 
-    # pylint: disable=too-many-arguments
+        return self._parse_job_ids(response.text)
+
+    # pylint: disable=too-many-arguments,too-many-positional-arguments, too-many-locals
     def transform(
         self,
         to_bck: "Bucket",
         etl_name: str,
         timeout: str = DEFAULT_ETL_TIMEOUT,
         prepend: str = "",
+        ext: Optional[Dict[str, str]] = None,
         continue_on_error: bool = False,
         dry_run: bool = False,
         force: bool = False,
         latest: bool = False,
         sync: bool = False,
+        num_workers: Optional[int] = None,
+        etl_pipeline: Optional[List[str]] = None,
     ):
         """
         Performs ETL operation on a list or range of objects in a bucket, placing the results in the destination bucket
@@ -245,12 +325,18 @@ class ObjectGroup(AISSource):
             etl_name (str): Name of existing ETL to apply
             timeout (str): Timeout of the ETL job (e.g. 5m for 5 minutes)
             prepend (str, optional): Value to prepend to the name of resulting transformed objects
+            ext (Dict[str, str], optional): Dict mapping each extension to the extension that will replace it
+                (i.e. {"jpg": "txt"})
             continue_on_error (bool, optional): Whether to continue if there is an error transforming a single object
             dry_run (bool, optional): Skip performing the transform and just log the intended actions
             force (bool, optional): Force this job to run over others in case it conflicts
                 (see "limited coexistence" and xact/xreg/xreg.go)
             latest (bool, optional): GET the latest object version from the associated remote bucket
             sync (bool, optional): synchronize destination bucket with its remote (e.g., Cloud or remote AIS) source
+            num_workers (int, optional): Number of concurrent workers (readers). Defaults to the number of target
+                mountpaths if omitted or zero. A value of -1 indicates no workers at all (i.e., single-threaded
+                execution). Any positive value will be adjusted not to exceed the number of target CPUs.
+            etl_pipeline (List[str], optional): List of ETL names to be used for the transformation pipeline.
 
         Raises:
             aistore.sdk.errors.AISError: All other types of errors with AIStore
@@ -275,17 +361,21 @@ class ObjectGroup(AISSource):
         copy_msg = CopyBckMsg(
             prepend=prepend, dry_run=dry_run, force=force, latest=latest, sync=sync
         )
-        transform_msg = TransformBckMsg(etl_name=etl_name, timeout=timeout)
+        transform_msg = TransformBckMsg(
+            etl_name=etl_name, timeout=timeout, etl_pipeline=etl_pipeline
+        )
         value = TCMultiObj(
             to_bck=to_bck.as_model(),
-            tc_msg=TCBckMsg(transform_msg=transform_msg, copy_msg=copy_msg),
+            tc_msg=TCBckMsg(ext=ext, transform_msg=transform_msg, copy_msg=copy_msg),
             object_selection=self._obj_collection.get_value(),
             continue_on_err=continue_on_error,
+            num_workers=num_workers,
         ).as_dict()
         return self.bck.make_request(
             HTTP_METHOD_POST, ACT_TRANSFORM_OBJECTS, value=value
         ).text
 
+    # pylint: disable=too-many-arguments,too-many-positional-arguments
     def archive(
         self,
         archive_name: str,
@@ -294,7 +384,7 @@ class ObjectGroup(AISSource):
         include_source_name: bool = False,
         allow_append: bool = False,
         continue_on_err: bool = False,
-    ):
+    ) -> List[str]:
         """
         Create or append to an archive
 
@@ -307,7 +397,7 @@ class ObjectGroup(AISSource):
             continue_on_err (bool, optional): Whether to continue if there is an error archiving a single object
 
         Returns:
-            Job ID (as str) that can be used to check the status of the operation
+            List[str]: List of job IDs that can be used to check the status of the operation
 
         """
         val = ArchiveMultiObj(
@@ -319,9 +409,12 @@ class ObjectGroup(AISSource):
             allow_append=allow_append,
             continue_on_err=continue_on_err,
         ).as_dict()
-        return self.bck.make_request(
+
+        response = self.bck.make_request(
             HTTP_METHOD_PUT, ACT_ARCHIVE_OBJECTS, value=val
-        ).text
+        )
+
+        return self._parse_job_ids(response.text)
 
     def list_names(self) -> List[str]:
         """
